//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Detector/DetectorItem.h
//! @brief     Defines class DetectorItem
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODEL_DETECTOR_DETECTORITEM_H
#define BORNAGAIN_GUI_MODEL_DETECTOR_DETECTORITEM_H

#include "GUI/Model/Descriptor/AxisProperty.h"
#include "GUI/Model/Detector/ResolutionFunctionItemCatalog.h"
#include "GUI/Model/Mask/MaskContainerItem.h"

class Beam;
class IDetector;
class IResolutionFunction2D;
class MaskContainerItem;
class ResolutionFunctionItem;

class DetectorItem {
public:
    DetectorItem();
    ~DetectorItem();

    std::unique_ptr<IDetector> createDetector() const;

    void importMasks(const MaskContainerItem* maskContainer);
    MaskContainerItem& maskItems();

    SelectionProperty<ResolutionFunctionItemCatalog>& resolutionFunctionSelection();

    template <typename T>
    T* setResolutionFunctionType();

    //! Scales the values provided by axes (to perform deg->rad conversion on the way to domain).
    double axesToCoreUnitsFactor() const { return 1.0; } // TODO rm

    void writeTo(QXmlStreamWriter* w) const;
    void readFrom(QXmlStreamReader* r);

    AxisProperty& phiAxis() { return m_phiAxis; }
    const AxisProperty& phiAxis() const { return m_phiAxis; }

    AxisProperty& alphaAxis() { return m_alphaAxis; }
    const AxisProperty& alphaAxis() const { return m_alphaAxis; }

    bool expandResolutionFunc = true;

private:
    std::unique_ptr<IResolutionFunction2D> createResolutionFunction() const;

    void addMasksToCore(IDetector* detector) const;

    //! for creation of domain detector; only filled and relevant in jobs
    MaskContainerItem m_maskContainerItem; // without serializtion
    SelectionProperty<ResolutionFunctionItemCatalog> m_resolutionFunction;

    AxisProperty m_phiAxis;
    AxisProperty m_alphaAxis;
};

template <typename T>
T* DetectorItem::setResolutionFunctionType()
{
    m_resolutionFunction.setCurrentItem<T>();
    return dynamic_cast<T*>(m_resolutionFunction.currentItem());
}

#endif // BORNAGAIN_GUI_MODEL_DETECTOR_DETECTORITEM_H
