//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Mask/MaskContainerModel.h
//! @brief     Defines MaskItems classes
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODEL_MASK_MASKCONTAINERMODEL_H
#define BORNAGAIN_GUI_MODEL_MASK_MASKCONTAINERMODEL_H

#include <QAbstractListModel>

class MaskContainerItem;
class MaskItem;
class MaskRoot;
class QModelIndex;
class QVariant;
class RegionOfInterestItem;

//! Provides interfaces to a MaskContainerItem, allowing its contents to be displayed and modified
//! using the Qt mechanisms.

class MaskContainerModel : public QAbstractListModel {
    Q_OBJECT
public:
    MaskContainerModel(MaskContainerItem* container);
    ~MaskContainerModel();

    //! Insert mask at given row.
    void insertMask(int row, MaskItem* maskItem);

    void maskToModel(MaskItem* maskItem);

    //! Remove a given mask
    void removeMask(MaskItem* maskItem);

    //! Remove all masks
    void clear();

    //! Copy masks from another MaskContainerItem
    void copy(const MaskContainerModel* src);

    // Begin overridden methods from QAbstractListModel
    QVariant data(const QModelIndex& index, int role) const override;
    int rowCount(const QModelIndex& parent) const override;
    // End overridden methods from QAbstractListModel

    void moveMask(int from_row, int to_row);
    void removeMaskAt(int row);

    QModelIndex indexOfItem(MaskRoot* item) const; // TODO: change this to MaskItem*
    MaskItem* itemForIndex(const QModelIndex& index) const;

    RegionOfInterestItem* regionOfInterestItem() const;

public:
    MaskContainerItem* maskContainer = nullptr;
};

#endif // BORNAGAIN_GUI_MODEL_MASK_MASKCONTAINERMODEL_H
